from OpenGL import GL,GLU
from vector import *

class Camera:
	def __init__(self, target):
		self.target = target
		self.distance = 10
		self.position = \
			Vector3(* self.target.ode_body.getPosition() ) + \
			self.distance * Vector3(* self.target.ode_body.vectorToWorld((0,-1,1)) )
		self.speed = Vector3(0,0,0)
	
	def apply_Mode1(self, TimeDelta):
		'Tries to keep a constant distance to the target'
		diff = Vector3(* self.target.ode_body.getPosition() ) - self.position
		self.speed += TimeDelta * (diff.Length()-self.distance) * diff.Normalized()
		
		eye = self.position
		lookat = Vector3(* self.target.ode_body.getPosition() )
		up = Vector3(0,0,1)
		return (eye,lookat,up)
	
	def apply_Mode2(self, TimeDelta):
		'Tries to always be in a specific position relative to the target'
		target_pos = Vector3(* self.target.ode_body.getPosition() )
		target_rel_pos = (Vector3(* self.target.ode_body.vectorToWorld((0,-1,0)) ) ) * self.distance
		self.position = target_pos + target_rel_pos
		diff = (target_pos + target_rel_pos) - self.position
		self.speed += TimeDelta * diff
		
		eye = self.position
		lookat = Vector3(* self.target.ode_body.getPosition() )
		up = Vector3(* self.target.ode_body.vectorToWorld(Vector3(0,0,1)) )
		return (eye,lookat,up)
	
	def apply(self,TimeDelta):
		self.position += TimeDelta * self.speed
		self.speed *= 2 ** (-TimeDelta)
		
		#pull the camera to the target
		(eye,lookat,up) = self.apply_Mode2(TimeDelta)
		
		GL.glLoadIdentity()
		GLU.gluLookAt(
			eye.x,   eye.y,   eye.z,
			lookat.x,lookat.y,lookat.z,
			up.x    ,up.y,    up.z)