import os

from model import Model
from sprite import Sprite,TextSprite
import Image
from pgu import gui

import level_data

data_py = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.normpath(os.path.join(data_py, '..', 'data'))

def filepath(filename):
	'''Determine the path to a file in the data directory.
	'''
	return os.path.join(data_dir, filename)

def get_level_image(level):
	return os.path.join(data_dir,"levels", "level%d.bmp" % level)

def load(filename, mode='rb'):
	return open(os.path.join(data_dir, filename), mode)

def load_theme(name):
	def theme_dir(name):
		return os.path.join(data_dir, "themes", name)
	if not name or name == "common":
		return gui.Theme([theme_dir("common")])
	else:
		return gui.Theme([theme_dir("common"),theme_dir(name)])
	
loaded_models = {}
def load_model(filename):
	global loaded_models
	if filename not in loaded_models:
		full_path = os.path.join(data_dir,"models",filename+".obj")
		loaded_models[filename] = Model(full_path)
	return loaded_models[filename]

loaded_sprites = {}
def load_sprite(filename):
	global loaded_sprites
	if filename not in loaded_sprites:
		full_path = os.path.join(data_dir,"sprites",filename)
		loaded_sprites[filename] = Sprite(Image.open(full_path))
	return loaded_sprites[filename]

loaded_text_sprites = {}
def load_text(filename):
	global loaded_text_sprites
	if filename not in loaded_text_sprites:
		full_image_path = os.path.join(data_dir,"fonts",filename+".png")
		full_data_path = os.path.join(data_dir,"fonts",filename+".txt")
		data = file(full_data_path,"r").readlines()
		data = map(lambda x: x.replace("\n",""), data[0:2],)
		data[1] = map(int,data[1].split(","))
		loaded_text_sprites[filename] = TextSprite(Image.open(full_image_path),data)
	return loaded_text_sprites[filename]

def load_image(filename):
	full_path = os.path.join(data_dir, filename)
	return Image.open(full_path)

def load_level(number, profile):
	return eval("level_data.Level%d" % number)(profile)
	
def CleanUp():
	global loaded_models, loaded_sprites, loaded_text_sprites
	loaded_models = {}
	loaded_sprites = {}
	loaded_text_sprites = {}