import array

from OpenGL import GL

from vector import *

class Ground:
	def __init__(self, heightmap_image):
		self.size = heightmap_image.size
		self.height = 10.0
		
		vertices = []
		heights = []
		winds = []
		for y in xrange(self.size[1]):
			for x in xrange(self.size[0]):
				height = heightmap_image.getpixel((x,y))[0]
				vertices.append(Vector3(
					x / float(self.size[0]-1),
					y / float(self.size[1]-1),
					height / float(255)))
				
				neigh = []
				for x2 in range(x-1,x+2):
					for y2 in range(y-1,y+2):
						try:
							neigh.append(heightmap_image.getpixel((x2,y2))[0])
						except:
							neigh.append(0)
				avg = sum(neigh)/float(len(neigh))
				
				wind = max(0, 80*(avg-height) )
				
				#winds.append(heightmap_image.getpixel((x,y))[1] / float(255))
				winds.append(wind / float(255))
				heights.append(height / float(255))
		self.vertices = vertices
		self.heights = heights
		self.winds = winds
		
		vertices_array = array.array("d")
		for v in vertices:
			vertices_array.fromlist(v.values)
		self.vertices_array = vertices_array.tostring()
		
		normals = []
		for y in xrange(self.size[1]):
			for x in xrange(self.size[0]):
				if x == 0:
					xdiff = heightmap_image.getpixel((x+1,y))[0] - heightmap_image.getpixel((x,y))[0]
				elif x == self.size[0]-1:
					xdiff = heightmap_image.getpixel((x,y))[0] - heightmap_image.getpixel((x-1,y))[0]
				else:
					xdiff = heightmap_image.getpixel((x+1,y))[0] - heightmap_image.getpixel((x-1,y))[0]
				xdiff *= self.height/255.0
				
				if y == 0:
					ydiff = heightmap_image.getpixel((x,y+1))[0] - heightmap_image.getpixel((x,y))[0]
				elif y == self.size[1]-1:
					ydiff = heightmap_image.getpixel((x,y))[0] - heightmap_image.getpixel((x,y-1))[0]
				else:
					ydiff = heightmap_image.getpixel((x,y+1))[0] - heightmap_image.getpixel((x,y-1))[0]
				ydiff *= self.height/255.0
				
				n = Vector3(1,0,xdiff).Cross(Vector3(0,1,ydiff))
				normals.append( n.Normalized() )
		
		normals_array = array.array("d")
		for n in normals:
			normals_array.fromlist(n.values)
		self.normals_array = normals_array.tostring()
		
		indices = array.array("H")
		for y in xrange(self.size[1]-1):
			for x in xrange(self.size[0]-1):
				indices.append(x*self.size[1]+y)
				indices.append(x*self.size[1]+y+1)
				indices.append((x+1)*self.size[1]+y+1)
				indices.append((x+1)*self.size[1]+y)
		self.indices = indices.tostring()
	
	def get_wind(self, position):
		x,y = int(position.x+0.5), int(position.y+0.5)
		array_pos = y*self.size[0] + x
		if array_pos < 0 or array_pos > len(self.winds):
			return 0
		else:
			return self.winds[array_pos]
	
	def get_height(self, position):
		x,y = int(position.x+0.5), int(position.y+0.5)
		array_pos = y*self.size[0] + x
		if array_pos < 0 or array_pos > len(self.winds):
			return 0
		else:
			return self.heights[array_pos] * self.height
	
	def draw(self):
		GL.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, (3.5,2.5,1.5,1.0))
		GL.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT, (0.2,0.15,0.10,1.0))
		
		GL.glDisable(GL.GL_TEXTURE_2D)
		GL.glEnable(GL.GL_LIGHTING)
		GL.glDisable(GL.GL_BLEND)
		
		GL.glPushMatrix()
		GL.glScalef(self.size[0]-1, self.size[1]-1, self.height)
		
		GL.glVertexPointer(3, GL.GL_DOUBLE, 0, self.vertices_array)
		GL.glNormalPointer(GL.GL_DOUBLE, 0, self.normals_array)
		GL.glEnableClientState(GL.GL_VERTEX_ARRAY)
		GL.glEnableClientState(GL.GL_NORMAL_ARRAY)
		GL.glDrawElements(
			GL.GL_QUADS,
			4*(self.size[0]-1)*(self.size[1]-1),
			GL.GL_UNSIGNED_SHORT,
			self.indices)
			
		GL.glPopMatrix()
		
