from OpenGL import GL,GLU
import ode
import math
import pygame
from pgu import gui

from widgets import *
from vector import Vector3
from screen import Screen
import sprite
import draw
import menus

#"ended" states, except None
LOSE = -1
WIN  = +1

class Level(Screen):
	def __init__(self, number, profile):
		Screen.__init__(self)
		self.number = number
		self.profile = profile
		self.paused = False
		self.stop_time = False
		self.ended = False
		self.time_limit = None
		
		self.plane = None
		self.camera = None
		self.ground = None
		self.checkpoints = None
		
		self.clock = Clock()
		self.heightmeter = HeightMeter()
		self.speedometer = SpeedOMeter()
		
		self.gui_end = sprite.GUISprite(600,450)
		c = gui.Color(value=(0,0,0,127), width = 600, height = 450)
		self.gui_end.add(c, 0, 0)
		l = gui.Label("Press space to exit..", color=(255,255,255))
		t = gui.Table(width = 600)
		t.td(l, align=0)
		self.gui_end.add(t, 0, 400)
		
		self.ode_world = ode.World()
	
	def AddEndMessage(self, text):
		f = pygame.font.SysFont(None,72)
		t = gui.Table(width = 600)
		l = gui.Label(text, color=(255,255,255), font = f)
		t.td(l, align=0)
		self.gui_end.add(t, 0, 100)
	
	def Win(self):
		self.stop_time = True
		if not self.ended:
			self.ended = WIN
			self.AddEndMessage("You have won!")
		self.profile.set_score(self.number, self.time_limit)
	
	def Lose(self):
		self.paused = True
		if not self.ended:
			self.ended = LOSE
			self.AddEndMessage("You have lost!")
	
	def Draw3D(self, TimeDelta):
		GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
		
		if self.camera:
			self.camera.apply(TimeDelta)
		
		GL.glLightfv(GL. GL_LIGHT0, GL.GL_POSITION, (1,-1,2,0))
		GL.glLightfv(GL. GL_LIGHT1, GL.GL_POSITION, (1,1,2,0))
		
		draw.BackGroundSphere.Get().draw()
		
		if self.plane:
			self.plane.draw()
		if self.ground:
			self.ground.draw()
		if self.checkpoints:
			for i in xrange(len(self.checkpoints)-1,-1,-1):
				self.checkpoints[i].draw(i)
	
	def MoveStuff(self, TimeDelta):
		if not self.paused:
			self.ode_world.step(TimeDelta)
			
			if self.plane:
				self.plane.move(TimeDelta, self.ground)
			if self.time_limit != None and not self.stop_time:
				self.time_limit -= TimeDelta
				if self.time_limit <= 0:
					self.time_limit = 0
					self.Lose()
		
		if self.checkpoints and len(self.checkpoints):
			if self.checkpoints[0].check_proximity(self.plane.ode_body.getPosition()):
				self.checkpoints = self.checkpoints[1:]
		
		if self.plane:
			if not self.plane.alive:
				self.Lose()
			if self.checkpoints == []:
				self.Win()
	
	def Draw2D(self, TimeDelta):
		if self.time_limit != None:
			self.clock.draw(self.time_limit)
			pass
		
		self.heightmeter.draw(self.plane.ode_body.getPosition()[2])
		self.speedometer.draw(Vector3(* self.plane.ode_body.getLinearVel() ).Length())
		
		GL.glTranslatef(100,100,0)
		if self.ended != False:
			self.gui_end.draw()
	
	def HandleEvent(self, event):
		Screen.HandleEvent(self,event)
		if event.type == pygame.KEYDOWN:
			if event.key == pygame.K_SPACE and self.ended != False:
				self.NextScreen = menus.MainMenu()
			elif event.key == pygame.K_p:
				print self.plane.ode_body.getPosition()
			elif event.key == pygame.K_ESCAPE:
				self.NextScreen = menus.MainMenu()
