import pygame
#from OpenGL import GL,GLU
from pgu import gui

import profile
from screen import GUIScreen

import data
import menus

class NewProfile(GUIScreen):
	def __init__(self):
		GUIScreen.__init__(self, theme = data.load_theme("newprofile"))
		self.profile_name = ""
		
		#background image
		i = gui.Image(data.filepath("Desktop1.jpg"))
		self.gui_sprite.add(i,0,0)
		
		#label
		l = gui.Label("Enter your name:")
		self.gui_sprite.add(l,155,130)
		
		#Name input
		i = gui.Input(value = "", size = 16)
		self.input = i
		t = gui.Table(width = 800)
		t.td(i, align=0)
		self.gui_sprite.add(t,0,200)
		
		#"Create" button
		b = gui.Button("Create Profile", width = 260, height=50)
		b.connect(gui.CLICK,self.Create)
		t = gui.Table(width = 800)
		t.td(b, align=0)
		self.gui_sprite.add(t,0,350)
		
		#"Cancel" button
		b = gui.Button("Cancel", width = 130, height=30)
		b.connect(gui.CLICK,self.Cancel)
		self.gui_sprite.add(b,640,550)
		
		#"Already exists" dialog
		f = pygame.font.SysFont(None,18)
		b = gui.Button("Okay", width = 160, height=50)
		b.connect(gui.CLICK,self.RemoveAlreadyExists)
		t = gui.Table(width = 40)
		label = gui.Label("The name already exists", color=(0,0,0))
		t.td(label, align=0)
		t.tr()
		t.td(b, align=0)
		size = t.resize(0,0)
		container = gui.Container(width = size[0]+30, height = size[1]+30)
		container.add(t,15,15)
		title = gui.Label("Error", font = f)
		self.already_exists_sprite = gui.Dialog(title,container)
	
	def Create(self):
		new_name = self.input.value.strip()
		if (new_name == ""):
			return
		if profile.Get().exists(new_name):
			self.ShowAlreadyExists()
			return
		profile.Get().add(profile.Profile(new_name))
		self.NextScreen = menus.MainMenu()
	
	def Cancel(self):
		self.NextScreen = SelectProfile()
	
	def ShowAlreadyExists(self):
		self.already_exists_sprite.open()
	
	def RemoveAlreadyExists(self):
		self.already_exists_sprite.close()

class SelectProfile(GUIScreen):	
	def __init__(self):
		GUIScreen.__init__(self, theme = data.load_theme("selectprofile"))
		
		#background image
		i = gui.Image(data.filepath("Desktop1.jpg"))
		self.gui_sprite.add(i,0,0)
		
		self.selected_name = profile.Get().current
		
		#label
		l = gui.Label("Select a profile:")
		self.gui_sprite.add(l,155,30)
		
		#Up/Down buttons
		b = gui.Button("", width = 30, height=70)
		b.connect(gui.CLICK,self.Up)
		self.gui_sprite.add(b,55,200)
		self.up_button = b
		
		b = gui.Button("", width = 30, height=70)
		b.connect(gui.CLICK,self.Down)
		self.gui_sprite.add(b,55,275)
		self.down_button = b
		
		#"New" button
		b = gui.Button("New Profile", width = 130, height=30)
		b.connect(gui.CLICK,self.NewProfile)
		self.gui_sprite.add(b,20,520)
		
		#"Select" button
		b = gui.Button("Delete Profile", width = 130, height=30)
		b.connect(gui.CLICK,self.DeleteProfile)
		self.gui_sprite.add(b,220,520)
		
		#"Select" button
		b = gui.Button("Select", width = 130, height=30)
		b.connect(gui.CLICK,self.SelectProfile)
		self.gui_sprite.add(b,600,520)
		
		#name list
		self.all_names = profile.Get().get_names()
		self.label_array = []
		self.label_count = min(8, len(self.all_names))
		self.current_position = 0
		group = gui.Group()
		self.radio_group = group
		group.connect(gui.CHANGE, self.ClickLabel)
		for i in range(self.label_count):
			s = gui.Radio(group,i)
			l = gui.Label("Text that is long enough to contain any possible name")
			self.label_array.append({"switch":s,"label":l})
		t = gui.Table()
		for i in range(self.label_count):
			t.td(self.label_array[i]["switch"])
			t.td(self.label_array[i]["label"])
			if i != self.label_count-1: t.tr()
		self.gui_sprite.add(t,130,100)
		self.ShowNames()
	
	def ShowNames(self):
		new_value = None
		for i in range(self.label_count):
			if i + self.current_position < len(self.all_names):
				self.label_array[i]["label"].value = self.all_names[i + self.current_position]
			else:
				self.label_array[i]["label"].value = ""
			if self.label_array[i]["label"].value == self.selected_name:
				new_value = i
			self.label_array[i]["label"].chsize()
		
		self.radio_group.value = new_value
		
		max_position = len(self.all_names) - self.label_count
		self.up_button.disabled = (self.current_position <= 0)
		self.down_button.disabled = (self.current_position >= max_position)
		
		self.gui_sprite.update()
	
	def Up(self):
		if self.current_position > 0:
			self.current_position -= 1
		self.ShowNames()
	def Down(self):
		max_position = len(self.all_names) - self.label_count
		if self.current_position < max_position:
			self.current_position += 1
		self.ShowNames()
	def ClickLabel(self):
		index = self.radio_group.value
		if isinstance(index,int):
			self.selected_name = self.label_array[index]["label"].value
		else:
			self.selected_name = None
		self.ShowNames()
	
	def NewProfile(self):
		self.NextScreen = NewProfile()
	def SelectProfile(self):
		if self.selected_name == None:
			return
		profile.Get().select(self.selected_name)
		self.NextScreen = menus.MainMenu()
	def DeleteProfile(self):
		if self.selected_name == None:
			return
		profile.Get().remove(self.selected_name)
		self.NextScreen = SelectProfile()
