from pgu import gui
import pygame

from screen import GUIScreen
import menuprofile
import data
import level_data
import profile

class LevelSelect(GUIScreen):
	def __init__(self):
		GUIScreen.__init__(self, theme = data.load_theme("mainmenu"))
		self.profile = profile.Get().get_current()
		
		#background image
		i = gui.Image(data.filepath("Desktop1.jpg"))
		self.gui_sprite.add(i,0,0)
		
		#label
		l = gui.Label("Select a profile:")
		self.gui_sprite.add(l,155,30)
		
		#Up/Down buttons
		b = gui.Button("", width = 100, height=30)
		b.connect(gui.CLICK,self.Up)
		self.gui_sprite.add(b,250,100)
		self.up_button = b
		
		b = gui.Button("", width = 100, height=30)
		b.connect(gui.CLICK,self.Down)
		self.gui_sprite.add(b,250,450)
		self.down_button = b
		
		#"Close" button
		b = gui.Button("Back", width = 130, height=30)
		b.connect(gui.CLICK,self.Close)
		self.gui_sprite.add(b,20,520)
		
		#level list
		self.all_names = level_data.LevelNames
		self.label_array = []
		self.label_count = min(3, len(self.all_names))
		self.current_position = 0
		for i in range(self.label_count):
			height = 160 + 100 * i
			
			b = gui.Button("", width = 80, height = 80)
			b.connect(gui.CLICK,self.Click,i)
			self.gui_sprite.add(b,60,height)
			
			l = gui.Label("Text that is long enough to contain any possible level description")
			self.gui_sprite.add(l,200,height+20)
			
			self.label_array.append({"button":b,"label":l})
		
		self.ShowNames()

	def ShowNames(self):
		for i in range(self.label_count):
			level = i + self.current_position
			if level < len(self.all_names):
				image = gui.Image(data.get_level_image(level+1))
				self.label_array[i]["button"].value = image
				self.label_array[i]["label"].value = self.all_names[level]
			else:
				self.label_array[i]["button"].value = ""
				self.label_array[i]["label"].value = ""
		
		max_position = len(self.all_names) - self.label_count
		self.up_button.disabled = (self.current_position <= 0)
		self.down_button.disabled = (self.current_position >= max_position)
		
		self.gui_sprite.update()
	
	def Up(self):
		if self.current_position > 0:
			self.current_position -= 1
		self.ShowNames()
	def Down(self):
		max_position = len(self.all_names) - self.label_count
		if self.current_position < max_position:
			self.current_position += 1
		self.ShowNames()
	
	def Close(self):
		self.NextScreen = MainMenu()
	
	def Click(self, index):
		level_number = index + self.current_position + 1
		self.NextScreen = data.load_level(level_number, self.profile)

class MainMenu(GUIScreen):
	def __init__(self):
		GUIScreen.__init__(self, theme = data.load_theme("mainmenu"))
		self.profile = profile.Get().get_current()
		
		#background image
		i = gui.Image(data.filepath("Desktop1.jpg"))
		self.gui_sprite.add(i,0,0)
		
		#Welcome label
		text = "Welcome, %s!" % self.profile.name
		l = gui.Label(text)
		self.gui_sprite.add(l,20,20)
		
		#Score label
		text = "Your score: %d" % self.profile.total_score()
		l = gui.Label(text)
		self.gui_sprite.add(l,520,20)
		
		#"Start" button
		b = gui.Button("Race", width = 300, height=50)
		b.connect(gui.CLICK,self.Race)
		self.gui_sprite.add(b,250,220)
		
		#"Garage" yet not implemented
		b = gui.Button("Garage", width = 300, height=50)
		self.gui_sprite.add(b,250,290)
		
		#"Rankings" button
		b = gui.Button("Rankings Board", width = 300, height=50)
		b.connect(gui.CLICK,self.ShowRankings)
		self.gui_sprite.add(b,250,360)
		
		#"Exit" button
		b = gui.Button("Exit", width = 300, height=50)
		b.connect(gui.CLICK,self.Exit)
		self.gui_sprite.add(b,250,430)
		
		#"Profiles" button
		b = gui.Button("Profiles", width = 130, height=50)
		b.connect(gui.CLICK,self.ShowProfiles)
		self.gui_sprite.add(b,20,65)
	
	def Race(self):
		self.NextScreen = LevelSelect()
	
	def ShowProfiles(self):
		self.NextScreen = menuprofile.SelectProfile()
	
	def ShowRankings(self):
		self.NextScreen = RankingsMenu()
	
	def Exit(self):
		self.NextScreen = 0

class RankingsMenu(GUIScreen):
	def __init__(self):
		GUIScreen.__init__(self, theme = data.load_theme("ranking"))
		
		#background image
		i = gui.Image(data.filepath("Desktop2.jpg"))
		self.gui_sprite.add(i,0,0)
		
		#create the score list
		score_names = []
		for i in range(10):
			score_names.append(("Player%d" % (i+1), 1000-100*i))
		all_names = profile.Get().get_names()
		for name in all_names:
			score_names.append((name,profile.Get().profiles[name].total_score()))
		score_names.sort(lambda x,y:cmp(x[1],y[1]), reverse = True)
		
		titles = [
			"Grand Champion",
			"Champion",
			"Minor Champion",
			"Second Champion",
			"Champion's Assistant",
			"Mega Master",
			"Master",
			"Specialist",
			"Second Specialist"]
		
		f = pygame.font.SysFont(None,32)
		for i in range(9):
			height = 240+i*30
			l = gui.Label(score_names[i][0], width = 250, color=(0,0,0), font = f)
			self.gui_sprite.add(l,90,height)
			l = gui.Label("%d"%score_names[i][1], width = 50, color=(0,0,0), font = f)
			self.gui_sprite.add(l,400,height)
			l = gui.Label(titles[i], width = 200, color=(0,0,0), font = f)
			self.gui_sprite.add(l,500,height)
		
		#"Close" button
		b = gui.Button("Is Nice..", width = 200, height=30)
		b.connect(gui.CLICK,self.Close)
		self.gui_sprite.add(b,300,530)
	
	def Close(self):
		self.NextScreen = MainMenu()