from OpenGL import GL
import ode
import pygame

from vector import *
import data

def line(from_point, to_point):
	GL.glDepthMask(0)
	GL.glDisable(GL.GL_LIGHTING)
	GL.glColor3f(1,1,1)
	GL.glLineWidth(3)
	GL.glBegin(GL.GL_LINES)
	GL.glVertex(*from_point)
	GL.glVertex(*to_point)
	GL.glEnd()
	GL.glColor3f(0,0,0)
	GL.glLineWidth(1)
	GL.glBegin(GL.GL_LINES)
	GL.glVertex(*from_point)
	GL.glVertex(*to_point)
	GL.glEnd()
	GL.glDepthMask(1)

class Plane:
	def __init__(self, ode_world):
		self.ode_body = ode.Body(ode_world)
		self.alive = True
	
	def draw(self):
		GL.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, (0.4,0.4,0.25,1.0))
		GL.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT, (0.6,0.6,0.6,1.0))
		
		GL.glDisable(GL.GL_TEXTURE_2D)
		GL.glEnable(GL.GL_LIGHTING)
		GL.glDisable(GL.GL_BLEND)
		
		GL.glPushMatrix()
		
		position = self.ode_body.getPosition()
		rotation = self.ode_body.getRotation()
		m = [0]*16
		for col in [0,1,2]:
			for row in [0,1,2]:
				m[col*4 + row] = rotation[col+row*3]
			m[3*4+col] = position[col]
		m[15] = 1
		GL.glMultMatrixd(m)
		
		self.model.draw()
		
		GL.glPopMatrix()
	
	def move(self, TimeDelta, ground):
		pass
	
	def explode(self):
		self.alive = False
		#Fireworks(self.position)

class Glider(Plane):
	def __init__(self,ode_world):
		Plane.__init__(self, ode_world)
		
		self.velocity = Vector3(0,1,0)
		self.gravity = 0.04
		
		self.model = data.load_model("Glider")
	
	def move(self, TimeDelta, ground):
		#gravity
		self.ode_body.addForce((0,0,-0.26))
		
		velocity = self.ode_body.getLinearVel()
		wind = ground.get_wind(Vector3(*self.ode_body.getPosition()))
		velocity_and_wind = (velocity[0],velocity[1],velocity[2]-wind)
		#del velocity, wind
		velocity = self.ode_body.vectorFromWorld(velocity)
		velocity_and_wind = self.ode_body.vectorFromWorld(velocity_and_wind)
		
		#lift
		LIFT_FACTOR = 0.0125
		self.ode_body.addRelForce((0,0,velocity_and_wind[1]*LIFT_FACTOR))
		
		#drag, glide
		DRAG_X = 0
		DRAG_Y = 0.001
		DRAG_Z = 1
		self.ode_body.addRelForce((
			-DRAG_X*velocity_and_wind[0],
			-DRAG_Y*velocity_and_wind[1],
			-DRAG_X*velocity_and_wind[2]))
		anguler_velocity = self.ode_body.getAngularVel()
		
		TORQUE_SLOWER = 3
		self.ode_body.addTorque((
			-TORQUE_SLOWER*anguler_velocity[0],
			-TORQUE_SLOWER*anguler_velocity[1],
			-TORQUE_SLOWER*anguler_velocity[2]))
		
		#winds
		if False:
			pos = Vector3(* self.ode_body.getPosition() )
			vec = Vector3(* self.ode_body.vectorFromWorld((0,0,1)) )
			vec2 = Vector3(* self.ode_body.getLinearVel() )
			line(pos,pos+vec2)
		
		#print "%+6.4f %+6.4f %+6.4f" % (velocity_and_wind[0],velocity_and_wind[1],velocity_and_wind[2])
		#velocity_and_wind = velocity
		self.ode_body.addRelForceAtRelPos((-0.5*velocity_and_wind[0],0,0),(0,-10,0))
		#WING_SIZE = 94.1
		WING_SIZE = 14.1
		self.ode_body.addRelForceAtRelPos((0,0,-WING_SIZE*velocity_and_wind[2]),(0,-0.2,0))

		#check if the plane is touching the ground
		ground_height = ground.get_height(Vector3(*self.ode_body.getPosition()))
		above_ground = self.ode_body.getPosition()[2] - ground_height
		if above_ground <= 0:
			self.explode()
		
		#print self.ode_body.getForce()
		PressedKeys = pygame.key.get_pressed()
		if PressedKeys[pygame.K_UP]:
			self.ode_body.addRelTorque((-2,0,0))
		if PressedKeys[pygame.K_DOWN]:
			self.ode_body.addRelTorque((+2,0,0))
		if PressedKeys[pygame.K_LEFT]:
			self.ode_body.addRelTorque((0,-2,0))
		if PressedKeys[pygame.K_RIGHT]:
			self.ode_body.addRelTorque((0,+2,0))