import pickle
import game
import data

PROFILES_FILE = "Profiles.data"

def LoadProfiles():
	try:
		file = data.load(PROFILES_FILE)
		ret_val = pickle.loads(file.read())
		if not isinstance(ret_val,Profiles):
			raise Exception
		return ret_val
	except Exception, e:
		return None

def Get():
	return game.Game.GetCurrent().profiles

class Profile:
	def __init__(self, name):
		self.name = name
		
		self.level_score = {}
		self.spent_score = 0
		
		self.gear = ["Glider"]
		self.key_mappings = {}
		
		self.selected_plane = "Glider"
		self.selected_gear = []
	
	def store(self):
		game.Game.GetCurrent().profiles.store()
	
	def available_levels(self):
		return max([0]+self.level_score.keys()) + 1
	
	def set_score(self, level, score):
		new_score = score
		if self.level_score.has_key(level):
			new_score = max(score, self.level_score[level])
		self.level_score[level] = new_score
		self.store()
	
	def total_score(self):
		return sum(self.level_score.values())
	
	def add_gear(self, gear_name, gear_price):
		if gear_name in self.gear:
			return False
		if gear_price > self.total_score() - self.spent_score:
			return False
		self.spent_score += gear_price
		self.gear.append(gear_name)
		self.store()
		return True

class Profiles:
	def __init__(self):
		self.profiles = {}
		self.current = None
	
	def select(self, name):
		if self.exists(name):
			self.current = name
		self.store()
	
	def exists(self, name):
		return self.profiles.has_key(name)
	
	def add(self, to_add):
		self.profiles[to_add.name] = to_add
		self.current = to_add.name
		self.store()
	
	def remove(self, to_remove_name):
		del self.profiles[to_remove_name]
		if self.current == to_remove_name:
			self.current = None
		self.store()
	
	def store(self):
		info = pickle.dumps(self)
		data_file = data.load(PROFILES_FILE,"wb")
		data_file.write(info)
	
	def get_current(self):
		if self.exists(self.current):
			return self.profiles[self.current]
		else:
			self.current = None
			return None
	
	def get_names(self):
		return self.profiles.keys()
